from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
from django.template import RequestContext
from main.models import UserBook, Book
from main.booksearch.googlebooksearchv2 import GoogleBookSearch, BookNotFound
from main.views.users import redirectToProfile
from main.forms import UserCreationForm
import logging

logger = logging.getLogger(__name__)

@login_required
def searchBooks(request, message=""):
    if 'searchfor' in request.GET:
        # Paging bits
        if 'page' in request.GET and int(request.GET['page']) > 0:
            page = int(request.GET['page'])
            showPrevious = True
        else:
            page = 0 
            showPrevious = False
        nextPage = page + 1
        previousPage = page - 1

        # Perform the actual search
        results = GoogleBookSearch().getBooks(request.GET['searchfor'], page=page)
        logger.debug("Found " + str(results.resultsFound) + " results")

        # Decorate the books with info about friends
        if request.user:
            friends = request.user.profile.getFriends()
            for result in results.entries:
                result['friendsWithBook'] = usersWithBook(request.user, friends, result['gid'])
                if UserBook.objects.filter(owner = request.user).filter(book__gid = result['gid']).count():
                    result['hasBook'] = True

        # More paging...
        if int(results.resultsFound) > 10:
            showNext = True
        else:
            showNext = False

        logger.debug("Found %s entries", len(results.entries))

        # Render it up
        templateDict = {'message': message,
                        'results': results,
                        'entries': results.entries,
                        'showPrevious': showPrevious,
                        'showNext': showNext,
                        'page': page,
                        'nextPage': nextPage,
                        'previousPage': previousPage,
                        'searchFor': request.GET['searchfor'],
                        'resultsFound': results.resultsFound}
        return render_to_response("searchresults.html", templateDict,RequestContext(request))
    else:
        logger.debug("No search term, showing blank screen")
        return render_to_response("searchresults.html", RequestContext(request))

@login_required
def addBookReturnToSearch(request):
    if 'gid' in request.GET and request.GET['gid']:
        newBook = request.user.profile.addBook(request.GET['gid'])
        if newBook:
            message = "Added " + newBook.book.title + " to your library"
            return searchBooks(request, message=message)
        else:
            return searchBooks(request, message="Book already added")
    else:
        return searchBooks(request)

def usersWithBook(user, userlist, gid):
    usersWithBook = []
    #get all existing userBooks for this book
    books = userBooksByGid(gid)
    if user and books:
        for friend in userlist:
            #check if the user is one of the owners in the list of userBooks
            #and if so, add them to the new list
            if books.filter(owner = friend['friend']).count():
                #todo: refactor so only the user id is added to the list
                #then do a filter of (users.id in [])
                usersWithBook.append(friend['friend'])
    return usersWithBook

def userBooksByGid(gid):
    if Book.objects.filter(gid=gid).count():
        actualBook = Book.objects.get(gid=gid)
        if UserBook.objects.filter(book=actualBook).count():
            return UserBook.objects.filter(book=actualBook)
        else:
            return False
    else:
        return False

@login_required
def bookDetails(request, gid):
    try:
        actualBook = Book.objects.lookupBook(gid=gid)

        templateData = {'actualBook': actualBook,
                        'searchedBook': GoogleBookSearch().getSingleBook(gid)}

        #check if the user has borrowed it from anyone
        borrowedFrom = request.user.profile.getBooksBorrowed().filter(book__gid=gid)
        if borrowedFrom.count():
            templateData['borrowedFrom'] = borrowedFrom

        #check if the user has requested it from anyone
        requestedFrom = request.user.profile.getBooksRequested().filter(userbook__book__gid=gid)
        if requestedFrom.count():
            templateData['requestedFrom'] = requestedFrom

        if request.user:
           hasBook = request.user.profile.getBooksOwned().filter(book__gid=gid).count()
           templateData['hasBook'] = hasBook
           if hasBook:
               templateData['bookOwned'] = request.user.profile.getBooksOwned().get(book__gid=gid)
               return render_to_response("books/ownedbookdetail.html",templateData,RequestContext(request))
           else:
                templateData['friendsWithBook'] = usersWithBook(request.user,request.user.profile.getFriends(),gid)
                return render_to_response("books/bookdetail.html",templateData,RequestContext(request))
    except BookNotFound as e:
        return render_to_response("books/booknotfound.html",RequestContext(request))

def testError(request):
    assert False
    pass

def homePage(request):
    if request.user.is_authenticated():
       return redirectToProfile(request)
    else:
        registrationForm=UserCreationForm()
        return render_to_response("static/home.html",{'registrationForm':registrationForm},RequestContext(request))
