from django.http import HttpResponse, HttpResponseRedirect
from django.template import Context,loader, RequestContext
from django.shortcuts import render_to_response
from django.contrib.auth.decorators import login_required
from friends.models import FriendshipInvitation
from main.forms import UserCreationForm, ChangeEmailForm
from django.contrib.auth import authenticate, logout, login
from django import forms
from datetime import date
from main.models import User,UserBook,Book, BorrowRequest, BorrowLocation
from bookgroups.models import Group
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
import csv, os, logging
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from main.utils import safeDictGet

log = logging.getLogger(__name__)

def register(request):
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            new_user=form.save()
            user = authenticate(username=request.POST["username"], password=request.POST["password1"])
            login(request,user)
            message = render_to_response ("emails/signupconfirmation.txt",{'user':user})
            send_mail("idle Library signup confirmation",message,"librarian@idlelibrary.com",[user.email])
            return redirectToProfile(request)
    else:
        form=UserCreationForm()
    return render_to_response("registration/register.html",
                              {'form':form},RequestContext(request))

@login_required
def profile(request, view="myoverview", additional={}):
    log.debug("displaying profile section:"+view)
    templateData={"booksOwned":bookStatus(request.user.profile.getBooksOwned()),
                    "booksBorrowed":bookStatus(request.user.profile.getBooksBorrowed()),
                    "booksLent":bookStatus(request.user.profile.getBooksLent()),
                    "currentPage":view}
    if view=="myfriends":
        templateData['groups'] = Group.objects.filter(users__id = request.user.id)
    
    for k,v in additional.iteritems():
        templateData[k]=v
    
    
    return render_to_response("profile/"+view+".html",templateData, RequestContext(request))

@login_required
def mybooks(request,page=1):
    log.debug("displaying mybooks view, page "+str(page))
    books=bookStatus(request.user.profile.getBooksOwned())
    pagedBooks = Paginator(books,10,orphans=3,allow_empty_first_page=False)
    try:
        pageOfBooks = pagedBooks.page(int(page))
    except (EmptyPage,InvalidPage):
	try:
            pageOfBooks = pagedBooks.page(pagedBooks.num_pages)
        except:
            pageOfBooks = None
    templateData={"booksOwned":pageOfBooks,
                  "currentPage":"mybooks",
                  "pagesCount":pagedBooks.num_pages
                 }
    return render_to_response("profile/mybooks.html",templateData, RequestContext(request))

@login_required
def showUserProfile(request, username, message=""):
    user=User.objects.get(username__exact=username)
    userProfile={"user":user, 
                 "relationship":request.user.profile.getfriendStatus(user)}
    if user.username == request.user.username:
        #if you try and view your own user profile, get redirected to your proper profile
        return redirectToProfile(request)
    else:
        
        #decide what view to show
        view=["books","friends"]
        if 'view' in request.GET and request.GET['view'] in view:
            template=request.GET['view']
        else:
            template="books"
        
        #check if the user has sent or been sent a friend request
        friendRequestRecieved=FriendshipInvitation.objects.filter(from_user=user, to_user=request.user).count()
        friendRequestSent=FriendshipInvitation.objects.filter(from_user=request.user, to_user=user).count()
        
        
        
        if template=="books":
           
            userProfile["userBooks"]=getPageOfUserbooks(page = int(safeDictGet(request.GET, 'page')),
                                                        userBooks = user.profile.getBooksOwned(),
                                                        orderby = safeDictGet(request.GET, 'orderby'),
                                                        user=user)
        
            

        templateData={"userProfile":userProfile ,
                      "validOrders":["title","author","recent"],
                      "currentPage":template,
                      "friendRequestRecieved":friendRequestRecieved,
                      'friendRequestSent':friendRequestSent,
                      'message':message}
        
        if template=="friends":
            templateData['groups'] = Group.objects.filter(users__id = user.id)
        
            
        if 'orderby' in request.GET:
            templateData["currentOrder"] = request.GET['orderby']
            
        return render_to_response("profile/userprofile/"+template+".html",templateData, RequestContext(request)) 

def getPageOfUserbooks(userBooks, user, page=1, orderby=""):
    #if an 'orderby' is specified, use it, otherwise use the default (recently added)
    if orderby=="title":
        userBooks = userBooks.order_by('book__title')
    elif orderby=="author":
        userBooks = userBooks.order_by('book__author')
    elif orderby=="recent":
        userBooks = userBooks.order_by('-book__id')  
    
    books=bookStatus(userBooks,user=user)
    
    if page==0:
        page=1
    #paginate the books, but only return the correct page
    pagedBooks = Paginator(books,10,orphans=3)
    try:
        pageOfBooks = pagedBooks.page(page)
    except (EmptyPage,InvalidPage):
        pageOfBooks = pagedBooks.page(pagedBooks.num_pages)
    return pageOfBooks

def bookStatus(books,user=""):
    bookList=[]
    for book in books:
        bookList.append(getBookStatus(book,user))
    return bookList

def getBookStatus(book,user=""):
    if book.returndue > date.today():
        days=book.returndue - date.today()
        overdue=False
    else:
        days=date.today() -book.returndue 
        overdue=True
        if user:
            if book.isLent==user:
                status="borrowed"
            elif BorrowRequest.objects.filter(requester=user,userbook=book).count():
                status="requested"
            else:
                status=""
        else:
            if book.isLent:
                status="lent"
            else:
                status=""
        if book.lentDate:
            timeLent=date.today()-book.lentDate
            daysLent=timeLent.days
        else:
            daysLent=0
        bookBorrowed={"book":book,
                "overdue":overdue,
                "days":days.days,
                "daysLent":daysLent,
                "status":status}
        return bookBorrowed
    
    
@login_required
def redirectToProfile(request, message=""):
    if message:
        request.user.message_set.create(message=message)
    return HttpResponseRedirect(reverse("profile"))
   
@login_required
def addBookToCurrentUser(request):
    if 'gid' in request.GET and request.GET['gid']:
        newBook=request.user.profile.addBook(request.GET['gid'])
        if newBook:
            profileMessage="added "+newBook.book.title +" to your library"
            return redirectToProfile(request,profileMessage)
        else:
            return profile(request,additional={'message':"book already added"})
    else:
        return redirectToProfile(request)




@login_required
def removeBookFromCurrentUser(request):
    if 'gid' in request.GET and request.GET['gid']:
        book=request.user.profile.removeBook(gid=request.GET['gid'])
        if book:
            message="deleted book:"+ book.title
        else:
            message=book.title+"wasn't in your profile"
        return render_to_response('profile/removebooks.html',{'message':message,'book':book},RequestContext(request))
    else:
        return redirectToProfile(request)

@login_required
def importBooksToCurrentUser(request):
    if request.POST:
        #have recieved data, import it
        books=csv.reader( request.POST['bookdata'].split( os.linesep ))
        message=""
        for book in books:
            bookAdded=request.user.profile.addBook(book[0])
            if bookAdded:
                message+="added "+bookAdded.book.title+", "
        return redirectToProfile(request,message)
    else:
        return render_to_response("profile/importbooksform.html", RequestContext(request)) 


#todo:actually finish this...
@login_required
def parseIsbnList(request):
    if request.POST and 'isbns' in request.POST:
        isbnsField=request.POST['isbns']
        isbnsField=isbns.replace(" ","")
        isbns=isbnsField.split(",")
        logger.log(isbns)
    else:
        return render_to_response("profile/importbooksform.html", RequestContext(request)) 
    
#todo:actually finish this...
def importGidListToCurrentUser(request):
    pass


@login_required
def listAllUsers(request):
    return render_to_response("allusers.html",{'userlist':User.objects.all().exclude(username=request.user.username)}, RequestContext(request))

@login_required
def changeEmailAddress(request):
    if request.method == "POST":
        changeEmailForm = ChangeEmailForm(request.user,request.POST)
        if changeEmailForm.is_valid():
            changeEmailForm.save()
            return render_to_response("profile/changeemailform.html",RequestContext(request))
    else:
        changeEmailForm = ChangeEmailForm(request.user)
    return render_to_response("profile/changeemailform.html",{'form': changeEmailForm},RequestContext(request))

@login_required
def showContact(request):
    if 'name' in request.GET and request.GET['name']:
        contact=BorrowLocation.objects.filter(relatedTo=request.user).filter(name=request.GET['name'])
        if contact.count():
            return render_to_response("contact.html",{'contact':contact[0]},RequestContext(request))
        else:
            render_to_response("contact.html",RequestContext(request))
    elif 'id' in request.GET and request.GET['id']:
        contact=BorrowLocation.objects.filter(relatedTo=request.user).filter(id=request.GET['id'])
        if contact.count():
            return render_to_response("contact.html",{'contact':contact[0]},RequestContext(request))
        else:
            render_to_response("contact.html",RequestContext(request))
    else:
        render_to_response("contact.html",RequestContext(request))
